function wl = wilks_lambda_change(X,Y,L)

% Change detection in dual polarization SAR data bmo Wilks' lambda
%
% wl = wilks_lambda_change(X,Y,L);
%
% Change detection in bi-temporal dual polarization SAR images
% (multi-look covariance representation) by means of Wilks' lambda
%
% Perform test for equality of two complex (1x1,) 2x2
% variance-covariance matrices representing
%
%   dual polarization
%   dual polarization, diagonal only
%   single polarization
%
% (to read the data from disk, see freadenvisar)
%
% ShhShh* ShhShv*     k    a
% ShvShh* ShvShv*  =  a*   ksi
%
% Input
%   X       - covariance matrix input at first time point;
%   Y       - covariance matrix input at second time point
%             X and Y must be 3-D with either
%               4 bands (dual polarization), or
%               2 bands (dual pol, diagonal only), or
%               1 band (power data);
%             reading order is alphabetical in indexes with
%             real parts before imaginary parts (as read by
%             freadenvisar), fx. for dual polarimetry:
%               hhhh, real(hhhv), imag(hhhv), hvhv
%   L       - number of looks for X and Y
%
% Output
%   struct wl with fields
%       Wilks(:,:,1) - |X|/|X+Y|
%       Wilks(:,:,2) - |Y|/|X+Y|
%       pBeta        - p-value for Beta distribution approximation for Wilk's Lambda
%       a            - alpha for Beta distribution (simulation based)
%       b            - beta for Beta distribution (simulation based)
%       L            - number of looks for X and Y

% (c) Copyright 2018
% Allan Aasbjerg Nielsen, PhD, MSc
% alan@dtu.dk, www.imm.dtu.dk/~alan
% 10-11 Nov 2018

if nargin<3
    help wilks_lambda_change
    error('wilks_lambda_change: wrong input');
end
if nargout>1
    help wilks_lambda_change
    error('wilks_lambda_change: wrong output');
end

pX = size(X);
pY = size(Y);
if ~all(pX==pY)
    error('wilks_lambda_change: X and Y must have same dimensions');
end
nrows = pX(1);
ncols = pX(2);
if  ndims(X)==3
    nvars = pX(3);
elseif ismatrix(X)
    nvars = 1;
else
    error('wilks_lambda_change: wrong sizes of input images');
end

if nvars==4 % dual polarimetry
    comment = 'dual polarization, experimental vales for Beta-dist a and b';
    fprintf('wilks_lambda_change: %s\n',comment)
    alpha = L*3/4 - 1/2; % simulation based
    beta = L*9/4;
elseif nvars==2 % dual polarimetry, diagonal only
    comment = 'dual polarization, diagonal only';   
    fprintf('wilks_lambda_change: %s\n',comment)    
    alpha = L*3/4; % simulation based
    beta = L*9/4;
elseif nvars==1 % one channel data only
    comment = 'single polarization';
    fprintf('wilks_lambda_change: %s\n',comment)    
    alpha = L; % simulation based
    beta = L;
elseif nvars==9
    error('wilks_lambda_change_change: full polarimetry not implemented');
elseif nvars==5
    error('wilks_lambda_change_change: 3x3 azimuthal symmetry not implemented');
elseif nvars==3
    error('wilks_lambda_change_change: 3x3 diagonal only not implemented');
else
    error('wilks_lambda_change_change: input must be (4, 2) or 1 band(s), real (and complex)');
end

det1 = wishart_det(X);
det2 = wishart_det(Y);
X = X+Y;
clear Y
det3 = wishart_det(X);
clear X

wl.Wilks = nan(nrows,ncols,2);
wl.Wilks(:,:,1) = det1./det3;
clear det1
wl.Wilks(:,:,2) = det2./det3;
clear det2 det3

% No theory, simulation based
wl.pBeta = cdf('Beta',wl.Wilks,alpha,beta);
wl.a = alpha;
wl.b = beta;
wl.L = L;
wl.Mode = comment;
wl.pBeta_comment ='p-value pBeta is simulation based, no theory, two-sided';
